@@ -1,21 +1,14 @@
-// Copyright (C) 2014-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
 #include <dhcp/dhcp4.h>
 #include <dhcp/option.h>
 #include <dhcp/option_int_array.h>
+#include <dhcp/option_vendor.h>
 #include <dhcpsrv/lease.h>
 #include <dhcp4/tests/dhcp4_client.h>
 #include <util/range_utilities.h>
@@ -30,7 +23,7 @@ namespace test {
 
 Dhcp4Client::Configuration::Configuration()
     : routers_(), dns_servers_(), log_servers_(), quotes_servers_(),
-      serverid_("0.0.0.0") {
+      serverid_("0.0.0.0"), siaddr_(asiolink::IOAddress::IPV4_ZERO_ADDRESS()) {
     reset();
 }
 
@@ -41,6 +34,9 @@ Dhcp4Client::Configuration::reset() {
     log_servers_.clear();
     quotes_servers_.clear();
     serverid_ = asiolink::IOAddress("0.0.0.0");
+    siaddr_ = asiolink::IOAddress::IPV4_ZERO_ADDRESS();
+    sname_.clear();
+    boot_file_name_.clear();
     lease_ = Lease4();
 }
 
@@ -57,7 +53,8 @@ Dhcp4Client::Dhcp4Client(const Dhcp4Client::State& state) :
     server_facing_relay_addr_("10.0.0.2"),
     srv_(boost::shared_ptr<NakedDhcpv4Srv>(new NakedDhcpv4Srv(0))),
     state_(state),
-    use_relay_(false) {
+    use_relay_(false),
+    circuit_id_() {
 }
 
 Dhcp4Client::Dhcp4Client(boost::shared_ptr<NakedDhcpv4Srv> srv,
@@ -75,7 +72,8 @@ Dhcp4Client::Dhcp4Client(boost::shared_ptr<NakedDhcpv4Srv> srv,
     server_facing_relay_addr_("10.0.0.2"),
     srv_(srv),
     state_(state),
-    use_relay_(false) {
+    use_relay_(false),
+    circuit_id_() {
 }
 
 void
@@ -169,14 +167,31 @@ Dhcp4Client::applyConfiguration() {
     Option4AddrLstPtr opt_log_servers = boost::dynamic_pointer_cast<
         Option4AddrLst>(resp->getOption(DHO_LOG_SERVERS));
     if (opt_log_servers) {
-        config_.log_servers_ = opt_routers->getAddresses();
+        config_.log_servers_ = opt_log_servers->getAddresses();
     }
     // Quotes Servers
     Option4AddrLstPtr opt_quotes_servers = boost::dynamic_pointer_cast<
         Option4AddrLst>(resp->getOption(DHO_COOKIE_SERVERS));
     if (opt_quotes_servers) {
-        config_.quotes_servers_ = opt_dns_servers->getAddresses();
+        config_.quotes_servers_ = opt_quotes_servers->getAddresses();
     }
+    // Vendor Specific options
+    OptionVendorPtr opt_vendor = boost::dynamic_pointer_cast<
+        OptionVendor>(resp->getOption(DHO_VIVSO_SUBOPTIONS));
+    if (opt_vendor) {
+        config_.vendor_suboptions_ = opt_vendor->getOptions();
+    }
+    // siaddr
+    config_.siaddr_ = resp->getSiaddr();
+    // sname
+    OptionBuffer buf = resp->getSname();
+    // sname is a fixed length field holding null terminated string. Use
+    // of c_str() guarantees that only a useful portion (ending with null
+    // character) is assigned.
+    config_.sname_.assign(std::string(buf.begin(), buf.end()).c_str());
+    // (boot)file
+    buf = resp->getFile();
+    config_.boot_file_name_.assign(std::string(buf.begin(), buf.end()).c_str());
     // Server Identifier
     OptionCustomPtr opt_serverid = boost::dynamic_pointer_cast<
         OptionCustom>(resp->getOption(DHO_DHCP_SERVER_IDENTIFIER));
@@ -212,6 +227,17 @@ Dhcp4Client::createMsg(const uint8_t msg_type) {
     return (msg);
 }
 
+void
+Dhcp4Client::appendExtraOptions() {
+    // If there are any custom options specified, add them all to the message.
+    if (!extra_options_.empty()) {
+        for (OptionCollection::iterator opt = extra_options_.begin();
+             opt != extra_options_.end(); ++opt) {
+            context_.query_->addOption(opt->second);
+        }
+    }
+}
+
 void
 Dhcp4Client::doDiscover(const boost::shared_ptr<IOAddress>& requested_addr) {
     context_.query_ = createMsg(DHCPDISCOVER);
@@ -228,6 +254,8 @@ Dhcp4Client::doDiscover(const boost::shared_ptr<IOAddress>& requested_addr) {
     if (ciaddr_.isSpecified()) {
         context_.query_->setCiaddr(ciaddr_.get());
     }
+    appendExtraOptions();
+
     // Send the message to the server.
     sendMsg(context_.query_);
     // Expect response.
@@ -247,6 +275,8 @@ Dhcp4Client::doInform(const bool set_ciaddr) {
     context_.query_ = createMsg(DHCPINFORM);
     // Request options if any.
     appendPRL();
+    // Any other options to be sent by a client.
+    appendExtraOptions();
     // The client sending a DHCPINFORM message has an IP address obtained
     // by some other means, e.g. static configuration. The lease which we
     // are using here is most likely set by the createLease method.
@@ -305,7 +335,7 @@ Dhcp4Client::doDecline() {
     // Include client identifier.
     appendClientId();
 
-    // Incluer server identifier.
+    // Include server identifier.
     appendServerId();
 
     // Remove configuration.
@@ -361,6 +391,8 @@ Dhcp4Client::doRequest() {
     appendName();
     // Include Client Identifier
     appendClientId();
+    // Any other options to be sent by a client.
+    appendExtraOptions();
     // Send the message to the server.
     sendMsg(context_.query_);
     // Expect response.
@@ -449,6 +481,8 @@ Dhcp4Client::receiveOneMsg() {
                               msg->getBuffer().getLength()));
     msg_copy->setRemoteAddr(msg->getLocalAddr());
     msg_copy->setLocalAddr(msg->getRemoteAddr());
+    msg_copy->setRemotePort(msg->getLocalPort());
+    msg_copy->setLocalPort(msg->getRemotePort());
     msg_copy->setIface(msg->getIface());
 
     msg_copy->unpack();
@@ -463,6 +497,15 @@ Dhcp4Client::sendMsg(const Pkt4Ptr& msg) {
         msg->setHops(1);
         msg->setGiaddr(relay_addr_);
         msg->setLocalAddr(server_facing_relay_addr_);
+        // Insert RAI
+        OptionPtr rai(new Option(Option::V4, DHO_DHCP_AGENT_OPTIONS));
+        // Insert circuit id, if specified.
+        if (!circuit_id_.empty()) {
+            rai->addOption(OptionPtr(new Option(Option::V4, RAI_OPTION_AGENT_CIRCUIT_ID,
+                                                OptionBuffer(circuit_id_.begin(),
+                                                             circuit_id_.end()))));
+        }
+        msg->addOption(rai);
     }
     // Repack the message to simulate wire-data parsing.
     msg->pack();
@@ -485,6 +528,11 @@ Dhcp4Client::setHWAddress(const std::string& hwaddr_str) {
     }
 }
 
+void
+Dhcp4Client::addExtraOption(const OptionPtr& opt) {
+    extra_options_.insert(std::make_pair(opt->getType(), opt));
+}
+
 } // end of namespace isc::dhcp::test
 } // end of namespace isc::dhcp
 } // end of namespace isc
