@@ -1,34 +1,31 @@
-// Copyright (C) 2013-2014  Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
 
 #include <asiolink/io_address.h>
 #include <cc/data.h>
-#include <config/ccsession.h>
+#include <cc/command_interpreter.h>
 #include <dhcp4/json_config_parser.h>
 #include <dhcp4/tests/dhcp4_test_utils.h>
+#include <dhcp/libdhcp++.h>
 #include <dhcp/option4_addrlst.h>
 #include <dhcp/option_int.h>
 #include <dhcp/option_int_array.h>
 #include <dhcp/option_custom.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcp/tests/iface_mgr_test_config.h>
+#include <dhcp/tests/pkt_captures.h>
+#include <dhcpsrv/cfg_db_access.h>
 #include <dhcpsrv/cfgmgr.h>
 #include <dhcpsrv/lease.h>
 #include <dhcpsrv/lease_mgr.h>
 #include <dhcpsrv/lease_mgr_factory.h>
+#include <log/logger_support.h>
+#include <stats/stats_mgr.h>
 
 using namespace std;
 using namespace isc::asiolink;
@@ -38,8 +35,30 @@ namespace isc {
 namespace dhcp {
 namespace test {
 
+BaseServerTest::BaseServerTest()
+    : original_datadir_(CfgMgr::instance().getDataDir()) {
+    CfgMgr::instance().setDataDir(TEST_DATA_BUILDDIR);
+}
+
+BaseServerTest::~BaseServerTest() {
+    // Remove default lease file.
+    std::ostringstream s2;
+    s2 << CfgMgr::instance().getDataDir() << "/" << "kea-leases4.csv";
+    static_cast<void>(::remove(s2.str().c_str()));
+
+    // Revert to original data directory.
+    CfgMgr::instance().setDataDir(original_datadir_);
+
+    // Revert to unit test logging, in case the test reconfigured it.
+    isc::log::initLogger();
+}
+
 Dhcpv4SrvTest::Dhcpv4SrvTest()
 :rcode_(-1), srv_(0) {
+
+    // Wipe any existing statistics
+    isc::stats::StatsMgr::instance().removeAll();
+
     subnet_ = Subnet4Ptr(new Subnet4(IOAddress("192.0.2.0"), 24, 1000,
                                      2000, 3000));
     pool_ = Pool4Ptr(new Pool4(IOAddress("192.0.2.100"), IOAddress("192.0.2.110")));
@@ -48,18 +67,28 @@ Dhcpv4SrvTest::Dhcpv4SrvTest()
     // Add Router option.
     Option4AddrLstPtr opt_routers(new Option4AddrLst(DHO_ROUTERS));
     opt_routers->setAddress(IOAddress("192.0.2.2"));
-    subnet_->getCfgOption()->add(opt_routers, false, "dhcp4");
+    subnet_->getCfgOption()->add(opt_routers, false, DHCP4_OPTION_SPACE);
 
     CfgMgr::instance().clear();
+    CfgMgr::instance().setFamily(AF_INET);
     CfgMgr::instance().getStagingCfg()->getCfgSubnets4()->add(subnet_);
     CfgMgr::instance().commit();
+
+    LibDHCP::clearRuntimeOptionDefs();
+
+    // Let's wipe all existing statistics.
+    isc::stats::StatsMgr::instance().removeAll();
 }
 
 Dhcpv4SrvTest::~Dhcpv4SrvTest() {
 
     // Make sure that we revert to default value
     CfgMgr::instance().clear();
-    CfgMgr::instance().echoClientId(true);
+
+    LibDHCP::clearRuntimeOptionDefs();
+
+    // Let's wipe all existing statistics.
+    isc::stats::StatsMgr::instance().removeAll();
 }
 
 void Dhcpv4SrvTest::addPrlOption(Pkt4Ptr& pkt) {
@@ -86,24 +115,24 @@ void Dhcpv4SrvTest::configureRequestedOptions() {
         option_dns_servers(new Option4AddrLst(DHO_DOMAIN_NAME_SERVERS));
     option_dns_servers->addAddress(IOAddress("192.0.2.1"));
     option_dns_servers->addAddress(IOAddress("192.0.2.100"));
-    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_dns_servers, false, "dhcp4"));
+    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_dns_servers, false, DHCP4_OPTION_SPACE));
 
     // domain-name
     OptionDefinition def("domain-name", DHO_DOMAIN_NAME, OPT_FQDN_TYPE);
     OptionCustomPtr option_domain_name(new OptionCustom(def, Option::V4));
     option_domain_name->writeFqdn("example.com");
-    subnet_->getCfgOption()->add(option_domain_name, false, "dhcp4");
+    subnet_->getCfgOption()->add(option_domain_name, false, DHCP4_OPTION_SPACE);
 
     // log-servers
     Option4AddrLstPtr option_log_servers(new Option4AddrLst(DHO_LOG_SERVERS));
     option_log_servers->addAddress(IOAddress("192.0.2.2"));
     option_log_servers->addAddress(IOAddress("192.0.2.10"));
-    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_log_servers, false, "dhcp4"));
+    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_log_servers, false, DHCP4_OPTION_SPACE));
 
     // cookie-servers
     Option4AddrLstPtr option_cookie_servers(new Option4AddrLst(DHO_COOKIE_SERVERS));
     option_cookie_servers->addAddress(IOAddress("192.0.2.1"));
-    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_cookie_servers, false, "dhcp4"));
+    ASSERT_NO_THROW(subnet_->getCfgOption()->add(option_cookie_servers, false, DHCP4_OPTION_SPACE));
 }
 
 void Dhcpv4SrvTest::messageCheck(const Pkt4Ptr& q, const Pkt4Ptr& a) {
@@ -134,23 +163,23 @@ Dhcpv4SrvTest::basicOptionsPresent(const Pkt4Ptr& pkt) {
     errmsg << "option missing in the response";
     if (!pkt->getOption(DHO_DOMAIN_NAME)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "domain-name " << errmsg));
+                                            << "domain-name " << errmsg.str()));
 
     } else if (!pkt->getOption(DHO_DOMAIN_NAME_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "dns-servers " << errmsg));
+                                            << "dns-servers " << errmsg.str()));
 
     } else if (!pkt->getOption(DHO_SUBNET_MASK)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "subnet-mask " << errmsg));
+                                            << "subnet-mask " << errmsg.str()));
 
     } else if (!pkt->getOption(DHO_ROUTERS)) {
         return (::testing::AssertionFailure(::testing::Message() << "routers "
-                                            << errmsg));
+                                            << errmsg.str()));
 
     } else if (!pkt->getOption(DHO_DHCP_LEASE_TIME)) {
         return (::testing::AssertionFailure(::testing::Message() <<
-                                            "dhcp-lease-time " << errmsg));
+                                            "dhcp-lease-time " << errmsg.str()));
 
     }
     return (::testing::AssertionSuccess());
@@ -163,23 +192,23 @@ Dhcpv4SrvTest::noBasicOptions(const Pkt4Ptr& pkt) {
     errmsg << "option present in the response";
     if (pkt->getOption(DHO_DOMAIN_NAME)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "domain-name " << errmsg));
+                                            << "domain-name " << errmsg.str()));
 
     } else if (pkt->getOption(DHO_DOMAIN_NAME_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "dns-servers " << errmsg));
+                                            << "dns-servers " << errmsg.str()));
 
     } else if (pkt->getOption(DHO_SUBNET_MASK)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "subnet-mask " << errmsg));
+                                            << "subnet-mask " << errmsg.str()));
 
     } else if (pkt->getOption(DHO_ROUTERS)) {
         return (::testing::AssertionFailure(::testing::Message() << "routers "
-                                            << errmsg));
+                                            << errmsg.str()));
 
     } else if (pkt->getOption(DHO_DHCP_LEASE_TIME)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "dhcp-lease-time " << errmsg));
+                                            << "dhcp-lease-time " << errmsg.str()));
 
     }
     return (::testing::AssertionSuccess());
@@ -191,11 +220,11 @@ Dhcpv4SrvTest::requestedOptionsPresent(const Pkt4Ptr& pkt) {
     errmsg << "option missing in the response";
     if (!pkt->getOption(DHO_LOG_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "log-servers " << errmsg));
+                                            << "log-servers " << errmsg.str()));
 
     } else if (!pkt->getOption(DHO_COOKIE_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "cookie-servers " << errmsg));
+                                            << "cookie-servers " << errmsg.str()));
 
     }
     return (::testing::AssertionSuccess());
@@ -207,11 +236,11 @@ Dhcpv4SrvTest::noRequestedOptions(const Pkt4Ptr& pkt) {
     errmsg << "option present in the response";
     if (pkt->getOption(DHO_LOG_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "log-servers " << errmsg));
+                                            << "log-servers " << errmsg.str()));
 
     } else if (pkt->getOption(DHO_COOKIE_SERVERS)) {
         return (::testing::AssertionFailure(::testing::Message()
-                                            << "cookie-servers " << errmsg));
+                                            << "cookie-servers " << errmsg.str()));
 
     }
     return (::testing::AssertionSuccess());
@@ -220,7 +249,7 @@ Dhcpv4SrvTest::noRequestedOptions(const Pkt4Ptr& pkt) {
 OptionPtr Dhcpv4SrvTest::generateClientId(size_t size /*= 4*/) {
 
     OptionBuffer clnt_id(size);
-    for (int i = 0; i < size; i++) {
+    for (size_t i = 0; i < size; i++) {
         clnt_id[i] = 100 + i;
     }
 
@@ -234,14 +263,14 @@ OptionPtr Dhcpv4SrvTest::generateClientId(size_t size /*= 4*/) {
 HWAddrPtr Dhcpv4SrvTest::generateHWAddr(size_t size /*= 6*/) {
     const uint8_t hw_type = 123; // Just a fake number (typically 6=HTYPE_ETHER, see dhcp4.h)
     OptionBuffer mac(size);
-    for (int i = 0; i < size; ++i) {
+    for (size_t i = 0; i < size; ++i) {
         mac[i] = 50 + i;
     }
     return (HWAddrPtr(new HWAddr(mac, hw_type)));
 }
 
 void Dhcpv4SrvTest::checkAddressParams(const Pkt4Ptr& rsp,
-                                       const SubnetPtr subnet,
+                                       const Subnet4Ptr subnet,
                                        bool t1_present,
                                        bool t2_present) {
 
@@ -331,7 +360,8 @@ void Dhcpv4SrvTest::checkServerId(const Pkt4Ptr& rsp, const OptionPtr& expected_
 
 void Dhcpv4SrvTest::checkClientId(const Pkt4Ptr& rsp, const OptionPtr& expected_clientid) {
 
-    bool include_clientid = CfgMgr::instance().echoClientId();
+    bool include_clientid =
+        CfgMgr::instance().getCurrentCfg()->getEchoClientId();
 
     // check that server included our own client-id
     OptionPtr opt = rsp->getOption(DHO_DHCP_CLIENT_IDENTIFIER);
@@ -388,7 +418,9 @@ Dhcpv4SrvTest::createPacketFromBuffer(const Pkt4Ptr& src_pkt,
     return (::testing::AssertionSuccess());
 }
 
-void Dhcpv4SrvTest::TearDown() {
+void
+// cppcheck-suppress unusedFunction
+Dhcpv4SrvTest::TearDown() {
 
     CfgMgr::instance().clear();
 
@@ -425,12 +457,12 @@ Dhcpv4SrvTest::testDiscoverRequest(const uint8_t msg_type) {
 
     // Initialize the source HW address.
     vector<uint8_t> mac(6);
-    for (int i = 0; i < 6; ++i) {
+    for (uint8_t i = 0; i < 6; ++i) {
         mac[i] = i * 10;
     }
     // Initialized the destination HW address.
     vector<uint8_t> dst_mac(6);
-    for (int i = 0; i < 6; ++i) {
+    for (uint8_t i = 0; i < 6; ++i) {
         dst_mac[i] = i * 20;
     }
     // Create a DHCP message. It will be used to simulate the
@@ -572,9 +604,13 @@ Dhcpv4SrvTest::configure(const std::string& config, const bool commit) {
 void
 Dhcpv4SrvTest::configure(const std::string& config, NakedDhcpv4Srv& srv,
                          const bool commit) {
-    ElementPtr json = Element::fromJSON(config);
+    ConstElementPtr json;
+    ASSERT_NO_THROW(json = parseJSON(config));
     ConstElementPtr status;
 
+    // Disable the re-detect flag
+    disableIfacesReDetect(json);
+
     // Configure the server and make sure the config is accepted
     EXPECT_NO_THROW(status = configureDhcp4Server(srv, json));
     ASSERT_TRUE(status);
@@ -582,12 +618,67 @@ Dhcpv4SrvTest::configure(const std::string& config, NakedDhcpv4Srv& srv,
     ConstElementPtr comment = config::parseAnswer(rcode, status);
     ASSERT_EQ(0, rcode);
 
+    // Use specified lease database backend.
+    ASSERT_NO_THROW( {
+        CfgDbAccessPtr cfg_db = CfgMgr::instance().getStagingCfg()->getCfgDbAccess();
+        cfg_db->setAppendedParameters("universe=4");
+        cfg_db->createManagers();
+    } );
+
     if (commit) {
         CfgMgr::instance().commit();
     }
  }
 
+Dhcpv4Exchange
+Dhcpv4SrvTest::createExchange(const Pkt4Ptr& query) {
+    return (Dhcpv4Exchange(srv_.alloc_engine_, query, srv_.selectSubnet(query)));
+}
 
+void
+Dhcpv4SrvTest::pretendReceivingPkt(NakedDhcpv4Srv& srv, const std::string& config,
+                                   uint8_t pkt_type, const std::string& stat_name) {
+
+    IfaceMgrTestConfig test_config(true);
+    IfaceMgr::instance().openSockets4();
+
+    // Apply the configuration we just received.
+    configure(config);
+
+    // Let's just use one of the actual captured packets that we have.
+    Pkt4Ptr pkt = PktCaptures::captureRelayedDiscover();
+
+    // We just need to tweak it a it, to pretend that it's type is as desired.
+    // Note that when receiving a packet, its on-wire form is stored in data_
+    // field. Most methods (including setType()) operates on option objects
+    // (objects stored in options_ after unpack() is called). Finally, outgoing
+    // packets are stored in out_buffer_. So we need to go through the full
+    // unpack/tweak/pack cycle and do repack, i.e. move the output buffer back
+    // to incoming buffer.
+    pkt->unpack();
+    pkt->setType(pkt_type); // Set message type.
+    pkt->pack();
+    pkt->data_.resize(pkt->getBuffer().getLength());
+    // Copy out_buffer_ to data_ to pretend that it's what was just received.
+    memcpy(&pkt->data_[0], pkt->getBuffer().getData(), pkt->getBuffer().getLength());
+
+    // Simulate that we have received that traffic
+    srv.fakeReceive(pkt);
+    srv.run();
+
+    using namespace isc::stats;
+    StatsMgr& mgr = StatsMgr::instance();
+    ObservationPtr pkt4_rcvd = mgr.getObservation("pkt4-received");
+    ObservationPtr tested_stat = mgr.getObservation(stat_name);
+
+    // All expected statistics must be present.
+    ASSERT_TRUE(pkt4_rcvd);
+    ASSERT_TRUE(tested_stat);
+
+    // They also must have expected values.
+    EXPECT_EQ(1, pkt4_rcvd->getInteger().first);
+    EXPECT_EQ(1, tested_stat->getInteger().first);
+}
 
 }; // end of isc::dhcp::test namespace
 }; // end of isc::dhcp namespace
