@@ -1,16 +1,8 @@
-// Copyright (C) 2011-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2015,2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #ifndef IFACE_MGR_H
 #define IFACE_MGR_H
@@ -66,7 +58,7 @@ public:
         isc::Exception(file, line, what) { };
 };
 
-/// @brief IfaceMgr exception thrown thrown when error occured during
+/// @brief IfaceMgr exception thrown thrown when error occurred during
 /// reading data from socket.
 class SocketReadError : public Exception {
 public:
@@ -74,8 +66,8 @@ public:
         isc::Exception(file, line, what) { };
 };
 
-/// @brief IfaceMgr exception thrown thrown when error occured during
-/// sedning data through socket.
+/// @brief IfaceMgr exception thrown thrown when error occurred during
+/// sending data through socket.
 class SocketWriteError : public Exception {
 public:
     SocketWriteError(const char* file, size_t line, const char* what) :
@@ -158,7 +150,7 @@ struct SocketInfo {
 /// may require use of fixed-size buffers. The size of such a buffer is
 /// returned by the OS kernel when the socket is opened. Hence, it is
 /// convenient to allocate the buffer when the socket is being opened and
-/// utilze it throughout the lifetime of the socket.
+/// utilize it throughout the lifetime of the socket.
 ///
 /// In order to avoid potentially expensive copies of the @c Iface objects
 /// holding pre-allocated buffers and multiple containers, this class is
@@ -311,7 +303,7 @@ public:
     /// @brief Check if the interface has the specified address assigned.
     ///
     /// @param address Address to be checked.
-    /// @return true if address is assigned to the intefrace, false otherwise.
+    /// @return true if address is assigned to the interface, false otherwise.
     bool hasAddress(const isc::asiolink::IOAddress& address) const;
 
     /// @brief Adds an address to an interface.
@@ -446,7 +438,7 @@ protected:
     /// Network interface name.
     std::string name_;
 
-    /// Interface index (a value that uniquely indentifies an interface).
+    /// Interface index (a value that uniquely identifies an interface).
     int ifindex_;
 
     /// List of assigned addresses.
@@ -732,7 +724,7 @@ public:
     /// (in microseconds)
     ///
     /// @throw isc::BadValue if timeout_usec is greater than one million
-    /// @throw isc::dhcp::SocketReadError if error occured when receiving a
+    /// @throw isc::dhcp::SocketReadError if error occurred when receiving a
     /// packet.
     /// @throw isc::dhcp::SignalInterruptOnSelect when a call to select() is
     /// interrupted by a signal.
@@ -754,7 +746,7 @@ public:
     /// (in microseconds)
     ///
     /// @throw isc::BadValue if timeout_usec is greater than one million
-    /// @throw isc::dhcp::SocketReadError if error occured when receiving a
+    /// @throw isc::dhcp::SocketReadError if error occurred when receiving a
     /// packet.
     /// @throw isc::dhcp::SignalInterruptOnSelect when a call to select() is
     /// interrupted by a signal.
@@ -885,7 +877,7 @@ public:
     /// @throw SocketOpenFailure if tried and failed to open socket.
     /// @return true if any sockets were open
     bool openSockets6(const uint16_t port = DHCP6_SERVER_PORT,
-                      IfaceMgrErrorMsgCallback error_handler = NULL);
+                      IfaceMgrErrorMsgCallback error_handler = 0);
 
     /// @brief Opens IPv4 sockets on detected interfaces.
     ///
@@ -954,7 +946,7 @@ public:
     /// @return true if any sockets were open
     bool openSockets4(const uint16_t port = DHCP4_SERVER_PORT,
                       const bool use_bcast = true,
-                      IfaceMgrErrorMsgCallback error_handler = NULL);
+                      IfaceMgrErrorMsgCallback error_handler = 0);
 
     /// @brief Closes all open sockets.
     /// Is used in destructor, but also from Dhcpv4Srv and Dhcpv6Srv classes.
@@ -1018,7 +1010,7 @@ public:
     /// @throw PacketFilterChangeDenied if there are open IPv4 sockets.
     void setPacketFilter(const PktFilterPtr& packet_filter);
 
-    /// @brief Set packet filter object to handle sending and receving DHCPv6
+    /// @brief Set packet filter object to handle sending and receiving DHCPv6
     /// messages.
     ///
     /// Packet filter objects provide means for the @c IfaceMgr to open sockets
@@ -1051,7 +1043,7 @@ public:
     /// implementation that supports this feature on a particular OS.
     /// If there isn't, the PktFilterInet object will be set. If the
     /// argument is set to 'false', PktFilterInet object instance will
-    /// be set as the Packet Filter regrdaless of the OS type.
+    /// be set as the Packet Filter regardless of the OS type.
     ///
     /// @param direct_response_desired specifies whether the Packet Filter
     /// object being set should support direct traffic to the host
@@ -1202,7 +1194,7 @@ private:
     bool openMulticastSocket(Iface& iface,
                              const isc::asiolink::IOAddress& addr,
                              const uint16_t port,
-                             IfaceMgrErrorMsgCallback error_handler = NULL);
+                             IfaceMgrErrorMsgCallback error_handler = 0);
 
     /// Holds instance of a class derived from PktFilter, used by the
     /// IfaceMgr to open sockets and send/receive packets through these
