@@ -1,4 +1,4 @@
-// Copyright (C) 2015 - 2016 Deutsche Telekom AG.
+// Copyright (C) 2015 - 2017 Deutsche Telekom AG.
 //
 // Author: Razvan Becheriu <razvan.becheriu@qualitance.com>
 //
@@ -563,7 +563,7 @@ public:
             // address: int
             // The address in the Lease structure is an IOAddress object.
             // Convert this to an integer for storage.
-            addr4_ = static_cast<uint32_t>(lease_->addr_);
+            addr4_ = lease_->addr_.toUint32();
             data.add(&addr4_);
 
             // hwaddr: blob
@@ -1387,7 +1387,7 @@ CqlLeaseMgr::getLease4(const isc::asiolink::IOAddress& addr) const {
     // Set up the WHERE clause value
     CqlDataArray data;
 
-    uint32_t addr4_data = static_cast<uint32_t>(addr);
+    uint32_t addr4_data = addr.toUint32();
     data.add(&addr4_data);
 
     // Get the data
@@ -1707,7 +1707,7 @@ CqlLeaseMgr::updateLease4(const Lease4Ptr& lease) {
     data.remove(0);
 
     // Set up the WHERE clause and append it to the SQL_BIND array
-    uint32_t addr4_data = static_cast<uint32_t>(lease->addr_);
+    uint32_t addr4_data = lease->addr_.toUint32();
     data.add(&addr4_data);
 
     // Drop to common update code
@@ -1805,7 +1805,7 @@ CqlLeaseMgr::deleteLease(const isc::asiolink::IOAddress& addr) {
     CqlDataArray data;
 
     if (addr.isV4()) {
-        uint32_t addr4_data = static_cast<uint32_t>(addr);
+        uint32_t addr4_data = addr.toUint32();
         data.add(&addr4_data);
         return (deleteLeaseCommon(DELETE_LEASE4, data, *exchange4_));
     } else {
@@ -1885,6 +1885,16 @@ CqlLeaseMgr::deleteExpiredReclaimedLeasesCommon(const uint32_t secs,
     return (result);
 }
 
+size_t
+CqlLeaseMgr::wipeLeases4(const SubnetID& /*subnet_id*/) {
+    isc_throw(NotImplemented, "wipeLeases4 is not implemented for Cassandra backend");
+}
+
+size_t
+CqlLeaseMgr::wipeLeases6(const SubnetID& /*subnet_id*/) {
+    isc_throw(NotImplemented, "wipeLeases6 is not implemented for Cassandra backend");
+}
+
 std::string
 CqlLeaseMgr::getName() const {
     std::string name = "";
@@ -1954,7 +1964,7 @@ CqlLeaseMgr::getVersion() const {
     cass_future_free(future);
     cass_statement_free(statement);
 
-    return make_pair<uint32_t, uint32_t>(version, minor);
+    return (make_pair(version, minor));
 }
 
 void
