@@ -1,21 +1,14 @@
-// Copyright (C) 2012-2014 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
 #include <dhcp/option4_client_fqdn.h>
 #include <dhcp/option6_client_fqdn.h>
 #include <dhcpsrv/d2_client_mgr.h>
+#include <testutils/test_to_element.h>
 #include <exceptions/exceptions.h>
 
 #include <gtest/gtest.h>
@@ -24,10 +17,39 @@ using namespace std;
 using namespace isc::asiolink;
 using namespace isc::dhcp;
 using namespace isc::util;
+using namespace isc::test;
 using namespace isc;
 
 namespace {
 
+/// @brief Tests conversion of NameChangeFormat between enum and strings.
+TEST(ReplaceClientNameModeTest, formatEnumConversion){
+    ASSERT_EQ(D2ClientConfig::stringToReplaceClientNameMode("never"),
+              D2ClientConfig::RCM_NEVER);
+    ASSERT_EQ(D2ClientConfig::stringToReplaceClientNameMode("always"),
+              D2ClientConfig::RCM_ALWAYS);
+    ASSERT_EQ(D2ClientConfig::stringToReplaceClientNameMode("when-present"),
+              D2ClientConfig::RCM_WHEN_PRESENT);
+    ASSERT_EQ(D2ClientConfig::stringToReplaceClientNameMode("when-not-present"),
+              D2ClientConfig::RCM_WHEN_NOT_PRESENT);
+    ASSERT_THROW(D2ClientConfig::stringToReplaceClientNameMode("BOGUS"),
+                 isc::BadValue);
+
+    ASSERT_EQ(D2ClientConfig::
+              replaceClientNameModeToString(D2ClientConfig::RCM_NEVER),
+              "never");
+    ASSERT_EQ(D2ClientConfig::
+              replaceClientNameModeToString(D2ClientConfig::RCM_ALWAYS),
+              "always");
+    ASSERT_EQ(D2ClientConfig::
+              replaceClientNameModeToString(D2ClientConfig::RCM_WHEN_PRESENT),
+              "when-present");
+    ASSERT_EQ(D2ClientConfig::
+              replaceClientNameModeToString(D2ClientConfig::
+                                            RCM_WHEN_NOT_PRESENT),
+              "when-not-present");
+}
+
 /// @brief Checks constructors and accessors of D2ClientConfig.
 TEST(D2ClientConfigTest, constructorsAndAccessors) {
     D2ClientConfigPtr d2_client_config;
@@ -55,7 +77,8 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
     bool always_include_fqdn = true;
     bool override_no_update = true;
     bool override_client_update = true;
-    bool replace_client_name = true;
+    D2ClientConfig::ReplaceClientNameMode replace_client_name_mode = D2ClientConfig::
+                                                                     RCM_WHEN_PRESENT;
     std::string generated_prefix = "the_prefix";
     std::string qualifying_suffix = "the.suffix.";
 
@@ -71,8 +94,8 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
                                                           ncr_format,
                                                           always_include_fqdn,
                                                           override_no_update,
-                                                         override_client_update,
-                                                          replace_client_name,
+                                                          override_client_update,
+                                                          replace_client_name_mode,
                                                           generated_prefix,
                                                           qualifying_suffix)));
 
@@ -92,7 +115,7 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
     EXPECT_EQ(d2_client_config->getOverrideNoUpdate(), override_no_update);
     EXPECT_EQ(d2_client_config->getOverrideClientUpdate(),
               override_client_update);
-    EXPECT_EQ(d2_client_config->getReplaceClientName(), replace_client_name);
+    EXPECT_EQ(d2_client_config->getReplaceClientNameMode(), replace_client_name_mode);
     EXPECT_EQ(d2_client_config->getGeneratedPrefix(), generated_prefix);
     EXPECT_EQ(d2_client_config->getQualifyingSuffix(), qualifying_suffix);
 
@@ -100,6 +123,25 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
     ASSERT_NO_THROW(std::cout << "toText test:" << std::endl <<
                     *d2_client_config << std::endl);
 
+    // Verify what toElement returns.
+    std::string expected = "{\n"
+        "\"enable-updates\": true,\n"
+        "\"server-ip\": \"127.0.0.1\",\n"
+        "\"server-port\": 477,\n"
+        "\"sender-ip\": \"127.0.0.1\",\n"
+        "\"sender-port\": 478,\n"
+        "\"max-queue-size\": 2048,\n"
+        "\"ncr-protocol\": \"UDP\",\n"
+        "\"ncr-format\": \"JSON\",\n"
+        "\"always-include-fqdn\": true,\n"
+        "\"override-no-update\": true,\n"
+        "\"override-client-update\": true,\n"
+        "\"replace-client-name\": \"when-present\",\n"
+        "\"generated-prefix\": \"the_prefix\",\n"
+        "\"qualifying-suffix\": \"the.suffix.\"\n"
+        "}\n";
+    runToElementTest<D2ClientConfig>(expected, *d2_client_config);
+
     // Verify that constructor does not allow use of NCR_TCP.
     /// @todo obviously this becomes invalid once TCP is supported.
     ASSERT_THROW(d2_client_config.reset(new
@@ -114,7 +156,7 @@ TEST(D2ClientConfigTest, constructorsAndAccessors) {
                                                        always_include_fqdn,
                                                        override_no_update,
                                                        override_client_update,
-                                                       replace_client_name,
+                                                       replace_client_name_mode,
                                                        generated_prefix,
                                                        qualifying_suffix)),
                  D2ClientError);
@@ -135,7 +177,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(ref_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(ref_config);
 
@@ -143,7 +185,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_TRUE(*ref_config == *test_config);
@@ -153,7 +195,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(false,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -163,7 +205,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     test_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -173,7 +215,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 333, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -183,7 +225,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, test_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -193,7 +235,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 333, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -203,7 +245,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 2048,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -213,7 +255,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    false, true, true, true,
+                    false, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -223,7 +265,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, false, true, true,
+                    true, false, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -233,7 +275,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, false, true,
+                    true, true, false, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -243,7 +285,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, false,
+                    true, true, true, D2ClientConfig::RCM_NEVER,
                     "pre-fix", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -253,7 +295,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "bogus", "suf-fix")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -263,7 +305,7 @@ TEST(D2ClientConfigTest, equalityOperator) {
     ASSERT_NO_THROW(test_config.reset(new D2ClientConfig(true,
                     ref_address, 477, ref_address, 478, 1024,
                     dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                    true, true, true, true,
+                    true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                     "pre-fix", "bogus")));
     ASSERT_TRUE(test_config);
     EXPECT_FALSE(*ref_config == *test_config);
@@ -308,7 +350,7 @@ TEST(D2ClientMgr, validConfig) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  true, true, true, true,
+                                  true, true, true, D2ClientConfig::RCM_WHEN_PRESENT,
                                   "pre-fix", "suf-fix")));
 
     // Verify that we can assign a new, non-empty configuration.
@@ -352,7 +394,7 @@ TEST(D2ClientMgr, analyzeFqdnInvalidCombination) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, false,
+                                  false, false, false, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
@@ -376,7 +418,7 @@ TEST(D2ClientMgr, analyzeFqdnEnabledNoOverrides) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, false,
+                                  false, false, false, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
@@ -420,7 +462,7 @@ TEST(D2ClientMgr, analyzeFqdnEnabledOverrideNoUpdate) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, true, false, false,
+                                  false, true, false, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
@@ -463,7 +505,7 @@ TEST(D2ClientMgr, analyzeFqdnEnabledOverrideClientUpdate) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
@@ -507,7 +549,7 @@ TEST(D2ClientMgr, adjustFqdnFlagsV4) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, true, false, false,
+                                  false, true, false, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
@@ -608,41 +650,41 @@ TEST(D2ClientMgr, qualifyName) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
 
     // Verify that the qualifying suffix gets appended with trailing dot added.
     std::string partial_name = "somehost";
-    std::string qualified_name = mgr.qualifyName(partial_name,true);
+    std::string qualified_name = mgr.qualifyName(partial_name, true);
     EXPECT_EQ("somehost.suffix.com.", qualified_name);
 
 
-	//append suffix but dot
+        //append suffix but dot
     ASSERT_NO_THROW(cfg.reset(new D2ClientConfig(true,
                                   isc::asiolink::IOAddress("127.0.0.1"), 477,
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     partial_name = "somehost";
-    qualified_name = mgr.qualifyName(partial_name,false); //false means no dot
+    qualified_name = mgr.qualifyName(partial_name, false); //false means no dot
     EXPECT_EQ("somehost.suffix.com", qualified_name);
 
 
-	//append no suffix and not dot
+        //append no suffix and not dot
     ASSERT_NO_THROW(cfg.reset(new D2ClientConfig(true,
                                   isc::asiolink::IOAddress("127.0.0.1"), 477,
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "prefix", ""))); //empty suffix
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     partial_name = "somehost";
-    qualified_name = mgr.qualifyName(partial_name,false); //false means no dot
+    qualified_name = mgr.qualifyName(partial_name, false); //false means no dot
     EXPECT_EQ("somehost", qualified_name);
 
     // Verify that the qualifying suffix gets appended with trailing dot added.
@@ -651,13 +693,49 @@ TEST(D2ClientMgr, qualifyName) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "prefix", "hasdot.com.")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
 
     // Verify that the qualifying suffix gets appended without dot added.
-    qualified_name = mgr.qualifyName(partial_name,true);
+    qualified_name = mgr.qualifyName(partial_name, true);
     EXPECT_EQ("somehost.hasdot.com.", qualified_name);
+
+    // Verify that the qualifying suffix gets appended without an
+    // extraneous dot when partial_name ends with a "."
+    qualified_name = mgr.qualifyName("somehost.", true);
+    EXPECT_EQ("somehost.hasdot.com.", qualified_name);
+
+    // Reconfigure to a "" suffix
+    ASSERT_NO_THROW(cfg.reset(new D2ClientConfig(true,
+                                  isc::asiolink::IOAddress("127.0.0.1"), 477,
+                                  isc::asiolink::IOAddress("127.0.0.1"), 478,
+                                  1024,
+                                  dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
+                                  "prefix", "")));
+    ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
+
+    // Verify that a name with a trailing dot does not get an extraneous
+    // dot when the suffix is blank
+    qualified_name = mgr.qualifyName("somehost.", true);
+    EXPECT_EQ("somehost.", qualified_name);
+
+    // Verify that a name with no trailing dot gets just a dot when the
+    // suffix is blank
+    qualified_name = mgr.qualifyName("somehost", true);
+    EXPECT_EQ("somehost.", qualified_name);
+
+    // Verify that a name with no trailing dot does not get dotted when the
+    // suffix is blank and trailing dot is false
+    qualified_name = mgr.qualifyName("somehost", false);
+    EXPECT_EQ("somehost", qualified_name);
+
+    // Verify that a name with trailing dot gets "undotted" when the
+    // suffix is blank and trailing dot is false
+    qualified_name = mgr.qualifyName("somehost.", false);
+    EXPECT_EQ("somehost", qualified_name);
+
 }
 
 
@@ -672,7 +750,7 @@ TEST(D2ClientMgr, generateFqdn) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, true, false,
+                                  false, false, true, D2ClientConfig::RCM_NEVER,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
 
@@ -706,13 +784,13 @@ TEST(D2ClientMgr, adjustDomainNameV4) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, false,
+                                  false, false, false, D2ClientConfig::RCM_NEVER,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
-    ASSERT_FALSE(cfg->getReplaceClientName());
+    ASSERT_EQ(D2ClientConfig::RCM_NEVER, cfg->getReplaceClientNameMode());
 
     // replace-client-name is false, client passes in empty fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option4ClientFqdn(0, Option4ClientFqdn::RCODE_CLIENT(),
                                         "", Option4ClientFqdn::PARTIAL));
     response.reset(new Option4ClientFqdn(*request));
@@ -749,13 +827,13 @@ TEST(D2ClientMgr, adjustDomainNameV4) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, true,
+                                  false, false, false, D2ClientConfig::RCM_WHEN_PRESENT,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
-    ASSERT_TRUE(cfg->getReplaceClientName());
+    ASSERT_EQ(D2ClientConfig::RCM_WHEN_PRESENT, cfg->getReplaceClientNameMode());
 
     // replace-client-name is true, client passes in empty fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option4ClientFqdn(0, Option4ClientFqdn::RCODE_CLIENT(),
                                         "", Option4ClientFqdn::PARTIAL));
     response.reset(new Option4ClientFqdn(*request));
@@ -765,7 +843,7 @@ TEST(D2ClientMgr, adjustDomainNameV4) {
     EXPECT_EQ(Option4ClientFqdn::PARTIAL, response->getDomainNameType());
 
     // replace-client-name is true, client passes in a partial fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option4ClientFqdn(0, Option4ClientFqdn::RCODE_CLIENT(),
                                         "myhost", Option4ClientFqdn::PARTIAL));
     response.reset(new Option4ClientFqdn(*request));
@@ -776,7 +854,7 @@ TEST(D2ClientMgr, adjustDomainNameV4) {
 
 
     // replace-client-name is true, client passes in a full fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option4ClientFqdn(0, Option4ClientFqdn::RCODE_CLIENT(),
                                         "myhost.example.com.",
                                          Option4ClientFqdn::FULL));
@@ -799,13 +877,13 @@ TEST(D2ClientMgr, adjustDomainNameV6) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, false,
+                                  false, false, false, D2ClientConfig::RCM_NEVER,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
-    ASSERT_FALSE(cfg->getReplaceClientName());
+    ASSERT_EQ(D2ClientConfig::RCM_NEVER, cfg->getReplaceClientNameMode());
 
     // replace-client-name is false, client passes in empty fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option6ClientFqdn(0, "", Option6ClientFqdn::PARTIAL));
     response.reset(new Option6ClientFqdn(*request));
 
@@ -839,13 +917,13 @@ TEST(D2ClientMgr, adjustDomainNameV6) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, false, false, true,
+                                  false, false, false, D2ClientConfig::RCM_WHEN_PRESENT,
                                   "prefix", "suffix.com")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
-    ASSERT_TRUE(cfg->getReplaceClientName());
+    ASSERT_EQ(D2ClientConfig::RCM_WHEN_PRESENT, cfg->getReplaceClientNameMode());
 
     // replace-client-name is true, client passes in empty fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option6ClientFqdn(0, "", Option6ClientFqdn::PARTIAL));
     response.reset(new Option6ClientFqdn(*request));
 
@@ -854,7 +932,7 @@ TEST(D2ClientMgr, adjustDomainNameV6) {
     EXPECT_EQ(Option6ClientFqdn::PARTIAL, response->getDomainNameType());
 
     // replace-client-name is true, client passes in a partial fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option6ClientFqdn(0, "myhost",
                                         Option6ClientFqdn::PARTIAL));
     response.reset(new Option6ClientFqdn(*request));
@@ -865,7 +943,7 @@ TEST(D2ClientMgr, adjustDomainNameV6) {
 
 
     // replace-client-name is true, client passes in a full fqdn
-    // reponse domain should be empty/partial.
+    // response domain should be empty/partial.
     request.reset(new Option6ClientFqdn(0, "myhost.example.com.",
                                         Option6ClientFqdn::FULL));
     response.reset(new Option6ClientFqdn(*request));
@@ -889,7 +967,7 @@ TEST(D2ClientMgr, adjustFqdnFlagsV6) {
                                   isc::asiolink::IOAddress("127.0.0.1"), 478,
                                   1024,
                                   dhcp_ddns::NCR_UDP, dhcp_ddns::FMT_JSON,
-                                  false, true, false, false,
+                                  false, true, false, D2ClientConfig::RCM_NEVER,
                                   "pre-fix", "suf-fix")));
     ASSERT_NO_THROW(mgr.setD2ClientConfig(cfg));
     ASSERT_TRUE(mgr.ddnsEnabled());
