@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -151,7 +151,7 @@ public:
     /// @param expected_code expected option code
     /// @param expected_repr expected representation (text, hex, exists)
     void checkTokenRelay6Option(const TokenPtr& token,
-                                uint8_t expected_level,
+                                int8_t expected_level,
                                 uint16_t expected_code,
                                 TokenOption::RepresentationType expected_repr) {
         ASSERT_TRUE(token);
@@ -173,8 +173,8 @@ public:
     /// @param exp_code expected option code to be parsed
     /// @param exp_repr expected representation to be parsed
     /// @param exp_tokens expected number of tokens
-    void testRelay6Option(std::string expr,
-                         uint8_t exp_level,
+    void testRelay6Option(const std::string& expr,
+                         int8_t exp_level,
                          uint16_t exp_code,
                          TokenOption::RepresentationType exp_repr,
                          int exp_tokens) {
@@ -189,13 +189,13 @@ public:
             return;
         }
 
-        // Parsing should succed and return a token.
+        // Parsing should succeed and return a token.
         EXPECT_TRUE(parsed_);
 
         // There should be the expected number of tokens.
         ASSERT_EQ(exp_tokens, eval.expression.size());
 
-        // checkt that the first token is TokenRelay6Option and that
+        // checked that the first token is TokenRelay6Option and that
         // is has the correct attributes
         checkTokenRelay6Option(eval.expression.at(0), exp_level, exp_code, exp_repr);
     }
@@ -212,7 +212,7 @@ public:
         EXPECT_EQ(type, pkt->getType());
     }
 
-    /// @brief Test that verifies access to the DHCP packet metadatas.
+    /// @brief Test that verifies access to the DHCP packet metadata.
     ///
     /// This test attempts to parse the expression, will check if the number
     /// of tokens is exactly as expected and then will try to verify if the
@@ -221,7 +221,7 @@ public:
     /// @param expr expression to be parsed
     /// @param exp_type expected metadata type to be parsed
     /// @param exp_tokens expected number of tokens
-    void testPktMetadata(std::string expr,
+    void testPktMetadata(const std::string& expr,
                          TokenPkt::MetadataType exp_type,
                          int exp_tokens) {
         EvalContext eval(Option::V6);
@@ -266,7 +266,7 @@ public:
     /// @param expr expression to be parsed
     /// @param exp_type expected field type to be parsed
     /// @param exp_tokens expected number of tokens
-    void testPkt4Field(std::string expr,
+    void testPkt4Field(const std::string& expr,
                        TokenPkt4::FieldType exp_type,
                        int exp_tokens) {
         EvalContext eval(Option::V4);
@@ -314,7 +314,8 @@ public:
     /// @param expr expression to be parsed
     /// @param exp_type expected field type to be parsed
     /// @param exp_tokens expected number of tokens
-    void testPkt6Field(std::string expr, TokenPkt6::FieldType exp_type,
+    void testPkt6Field(const std::string& expr,
+                       TokenPkt6::FieldType exp_type,
                        int exp_tokens) {
         EvalContext eval(Option::V6);
 
@@ -344,7 +345,7 @@ public:
     /// @param expected_code expected option code
     /// @param expected_repr expected representation (text, hex, exists)
     void checkTokenRelay6Field(const TokenPtr& token,
-                               uint8_t expected_level,
+                               int8_t expected_level,
                                TokenRelay6Field::FieldType expected_type) {
         ASSERT_TRUE(token);
         boost::shared_ptr<TokenRelay6Field> opt =
@@ -363,8 +364,8 @@ public:
     /// @param exp_level expected level to be parsed
     /// @param exp_type expected field type to be parsed
     /// @param exp_tokens expected number of tokens
-    void testRelay6Field(std::string expr,
-                         uint8_t exp_level,
+    void testRelay6Field(const std::string& expr,
+                         int8_t exp_level,
                          TokenRelay6Field::FieldType exp_type,
                          int exp_tokens) {
         EvalContext eval(Option::V6);
@@ -378,13 +379,13 @@ public:
             return;
         }
 
-        // Parsing should succed and return a token.
+        // Parsing should succeed and return a token.
         EXPECT_TRUE(parsed_);
 
         // There should be the expected number of tokens.
         ASSERT_EQ(exp_tokens, eval.expression.size());
 
-        // checkt that the first token is TokenRelay6Field and that
+        // checked that the first token is TokenRelay6Field and that
         // is has the correct attributes
         checkTokenRelay6Field(eval.expression.at(0), exp_level, exp_type);
     }
@@ -461,8 +462,9 @@ public:
     /// @param vendor_id expected vendor-id (aka enterprise number)
     /// @param option_code expected option code (ignored if 0)
     /// @param expected_repr expected representation (either 'exists' or 'hex')
-    void testVendor(std::string expr, Option::Universe u, uint32_t vendor_id,
-                    uint16_t option_code, TokenOption::RepresentationType expected_repr) {
+    void testVendor(const std::string& expr, Option::Universe u,
+                    uint32_t vendor_id, uint16_t option_code,
+                    TokenOption::RepresentationType expected_repr) {
         EvalContext eval(u);
 
         EXPECT_NO_THROW(parsed_ = eval.parseString(expr));
@@ -487,7 +489,8 @@ public:
     /// @param u universe (V4 or V6)
     /// @param vendor_id expected vendor-id (aka enterprise number)
     /// @param expected_repr expected representation (either 'exists' or 'hex')
-    void testVendor(std::string expr, Option::Universe u, uint32_t vendor_id,
+    void testVendor(const std::string& expr, Option::Universe u,
+                    uint32_t vendor_id,
                     TokenOption::RepresentationType expected_repr) {
         testVendor(expr, u, vendor_id, 0, expected_repr);
     }
@@ -499,7 +502,8 @@ public:
     ///
     /// @param expr expression to be parsed
     /// @param u universe (V4 or V6)
-    void testVendorEnterprise(std::string expr, Option::Universe u) {
+    void testVendorEnterprise(const std::string& expr,
+                              Option::Universe u) {
         EvalContext eval(u);
 
         EXPECT_NO_THROW(parsed_ = eval.parseString(expr));
@@ -524,7 +528,8 @@ public:
     /// @param expr expression to be parsed
     /// @param u universe (V4 or V6)
     /// @param vendor_id expected vendor-id (aka enterprise number)
-    void testVendorClass(std::string expr, Option::Universe u, uint32_t vendor_id) {
+    void testVendorClass(const std::string& expr,
+                         Option::Universe u, uint32_t vendor_id) {
         EvalContext eval(u);
 
         EXPECT_NO_THROW(parsed_ = eval.parseString(expr));
@@ -548,8 +553,8 @@ public:
     /// @param u universe (V4 or V6)
     /// @param vendor_id expected vendor-id (aka enterprise number)
     /// @param index expected data index
-    void testVendorClass(std::string expr, Option::Universe u, uint32_t vendor_id,
-                         uint16_t index) {
+    void testVendorClass(const std::string& expr, Option::Universe u,
+                         uint32_t vendor_id, uint16_t index) {
         EvalContext eval(u);
 
         EXPECT_NO_THROW(parsed_ = eval.parseString(expr));
@@ -572,7 +577,8 @@ public:
     ///
     /// @param expr expression to be parsed
     /// @param u universe (V4 or V6)
-    void testVendorClassEnterprise(std::string expr, Option::Universe u) {
+    void testVendorClassEnterprise(const std::string& expr,
+                                   Option::Universe u) {
         EvalContext eval(u);
 
         EXPECT_NO_THROW(parsed_ = eval.parseString(expr));
@@ -928,7 +934,15 @@ TEST_F(EvalContextTest, relay6OptionHex) {
                      2, 85, TokenOption::HEXADECIMAL, 3);
 }
 
-// Test the nest level of a relay6 option should be in [0..32[
+// Test the parsing of a relay6 option in reverse order
+TEST_F(EvalContextTest, relay6OptionReverse) {
+    EvalContext eval(Option::V6);
+
+    testRelay6Option("relay6[-1].option[123].text == 'foo'",
+                     -1, 123, TokenOption::TEXTUAL, 3);
+}
+
+// Test the nest level of a relay6 option should be in [-32..32[
 TEST_F(EvalContextTest, relay6OptionLimits) {
     EvalContext eval(Option::V6);
 
@@ -940,11 +954,14 @@ TEST_F(EvalContextTest, relay6OptionLimits) {
 
     checkError("relay6[32].option[123].text == 'foo'",
                "<string>:1.8-9: Nest level has invalid value in 32. "
-               "Allowed range: 0..31");
+               "Allowed range: -32..31");
 
-    // next level must be a positive number
-    checkError("relay6[-1].option[123].text == 'foo'",
-               "<string>:1.8-9: Invalid value in -1. Allowed range: 0..255");
+    // min nest level is minus hop count limit
+    testRelay6Option("relay6[-32].option[123].text == 'foo'",
+                     -32, 123, TokenOption::TEXTUAL, 3);
+
+    checkError("relay6[-33].option[123].text == 'foo'",
+               "<string>:1.8-10: Nest level has invalid value in -33. Allowed range: -32..31");
 }
 
 // Verify that relay6[13].option is not usable in v4
@@ -1035,7 +1052,7 @@ TEST_F(EvalContextTest, relay6FieldPeerAddr) {
                     1, TokenRelay6Field::PEERADDR, 3);
 }
 
-// Verify that relay6[13].<field> is not usable in v4
+// Verify that relay6[0].<field> is not usable in v4
 TEST_F(EvalContextTest, relay6FieldError) {
     universe_ = Option::V4;
 
@@ -1201,7 +1218,10 @@ TEST_F(EvalContextTest, scanErrors) {
     checkError("0x123h", "<string>:1.6: Invalid character: h");
     checkError(":1", "<string>:1.1: Invalid character: :");
     checkError("=", "<string>:1.1: Invalid character: =");
+
+    // Typo should be handled as well.
     checkError("subtring", "<string>:1.1: Invalid character: s");
+
     checkError("foo", "<string>:1.1: Invalid character: f");
     checkError(" bar", "<string>:1.2: Invalid character: b");
     checkError("relay[12].hex == 'foo'", "<string>:1.1: Invalid character: r");
@@ -1353,7 +1373,7 @@ TEST_F(EvalContextTest, typeErrors) {
                "hexstring, expecting integer");
 
     // With the #4483 addition, all integers are treated as 4 byte strings,
-    // so those checks no longer makes sense. Commeting it out.
+    // so those checks no longer makes sense. Commenting it out.
     // checkError("concat('foo',3) == 'foo3'",
     //            "<string>:1.14: syntax error, unexpected integer");
     // checkError("concat(3,'foo') == '3foo'",
