@@ -93,6 +93,42 @@ public:
         EXPECT_EQ(r3, result) << "hookpt_three" << COMMON_TEXT;
     }
 
+    /// @brief Call command handlers test.
+    ///
+    /// This test is similar to @c executeCallCallouts but it uses
+    /// @ref HooksManager::callCommandHandlers to execute the command
+    /// handlers for the following commands: 'command-one' and 'command-two'.
+    ///
+    /// @param r1..r2, d1..d2 Data (dN) and expected results (rN).
+    void executeCallCommandHandlers(int d1, int r1, int d2, int r2) {
+        static const char* COMMON_TEXT = " command handler returned the wrong value";
+        static const char* RESULT = "result";
+
+        int result;
+
+        // Set up a callout handle for the calls.
+        CalloutHandlePtr handle = HooksManager::createCalloutHandle();
+
+        // Initialize the argument RESULT.  This simplifies testing by
+        // eliminating the generation of an exception when we try the unload
+        // test.  In that case, RESULT is unchanged.
+        handle->setArgument(RESULT, -1);
+
+        // Perform the first calculation: it should assign the data to the
+        // result.
+        handle->setArgument("data_1", d1);
+        HooksManager::callCommandHandlers("command-one", *handle);
+        handle->getArgument(RESULT, result);
+        EXPECT_EQ(d1, result) << "command-one" << COMMON_TEXT;
+
+        // Perform the second calculation: it should multiply the data by 10
+        // and return in the result.
+        handle->setArgument("data_2", d2);
+        HooksManager::callCommandHandlers("command-two", *handle);
+        handle->getArgument(RESULT, result);
+        EXPECT_EQ(r2, result) << "command-two" << COMMON_TEXT;
+    }
+
 private:
     /// To avoid unused variable errors
     std::string dummy(int i) {
@@ -138,6 +174,12 @@ TEST_F(HooksManagerTest, LoadLibraries) {
         executeCallCallouts(10, 3, 33, 2, 62, 3, 183);
     }
 
+    // r2 = 5 * 7 * 10
+    {
+        SCOPED_TRACE("Calculation using command handlers");
+        executeCallCommandHandlers(5, 5, 7, 350);
+    }
+
     // Try unloading the libraries.
     EXPECT_NO_THROW(HooksManager::unloadLibraries());
 
@@ -431,7 +473,7 @@ TEST_F(HooksManagerTest, PrePostCalloutShared) {
 
     HooksManager::callCallouts(hookpt_two_index_, *handle);
 
-    // Expect same value i.e. 1027 * 2 
+    // Expect same value i.e. 1027 * 2
     result = 0;
     handle->getArgument("result", result);
     EXPECT_EQ(2054, result);
@@ -538,6 +580,8 @@ TEST_F(HooksManagerTest, NoLibrariesCalloutsPresent) {
     EXPECT_FALSE(HooksManager::calloutsPresent(hookpt_one_index_));
     EXPECT_FALSE(HooksManager::calloutsPresent(hookpt_two_index_));
     EXPECT_FALSE(HooksManager::calloutsPresent(hookpt_three_index_));
+    EXPECT_FALSE(HooksManager::commandHandlersPresent("command-one"));
+    EXPECT_FALSE(HooksManager::commandHandlersPresent("command-two"));
 }
 
 TEST_F(HooksManagerTest, NoLibrariesCallCallouts) {
@@ -564,8 +608,13 @@ TEST_F(HooksManagerTest, RegisterHooks) {
     EXPECT_EQ(2, HooksManager::registerHook(string("alpha")));
     EXPECT_EQ(3, HooksManager::registerHook(string("beta")));
     EXPECT_EQ(4, HooksManager::registerHook(string("gamma")));
-    EXPECT_THROW(static_cast<void>(HooksManager::registerHook(string("alpha"))),
-                 DuplicateHook);
+
+
+    // The code used to throw, but it now allows to register the same
+    // hook several times. It simply returns existing index.
+    //EXPECT_THROW(static_cast<void>(HooksManager::registerHook(string("alpha"))),
+    //             DuplicateHook);
+    EXPECT_EQ(2, HooksManager::registerHook(string("alpha")));
 
     // ... an check the hooks are as we expect.
     EXPECT_EQ(5, ServerHooks::getServerHooks().getCount());
